;
;   This is part of servo.
;   Phisical access to I/O microcontroller lines.
;   Copyright (C) 2006 2008 2012  Alessandro Accattini
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
;*****************************************************************************
;
;
;

.CSEG

IoInit:
;Inizializza le porte di I/O impostando la direzione dei bit e lo
;stato di default delle uscite. 
;In uscita:	r16 viene perso
		ser r16			;imposta la porta B
		out DDRB,r16
		clr r16
		out PORTB,r16
		ldi r16,0xfe		;imposta la porta D
		out DDRD,r16
		com r16
		out PORTD,r16
		ret			;termina


;I pin di controllo dei servomotori sono raggruppati in 6 gruppi di 2 servi
;ciascuno, l'array ServoPin contiene 4 byte per ogni gruppo, per poter chiamare
;le funzioni RisePins, FallPin0 e FallPin1, che gestiscono ripettivamente: la
;salita di entrambi i pin di controllo, la discesa del primo e la discesa del
;secondo del gruppo; Z deve puntare al primo byte del gruppo attivo.

.equ	NONE	= 0xff
.equ	PIN0	= 0xfe
.equ	PIN1	= 0xfd
.equ	PIN2	= 0xfb
.equ	PIN3	= 0xf7
.equ	PIN4	= 0xef
.equ	PIN5	= 0xdf
.equ	PIN6	= 0xbf
.equ	PIN7	= 0x7f

;		PORT: B    D    B    D
ServoPin:	.db NONE,PIN2,PIN7,NONE	;servo 0=PD2	1=PB7
		.db PIN5,NONE,PIN3,NONE	;servo 0=PB5	1=PB3
		.db PIN1,NONE,NONE,PIN4	;servo 0=PB1	1=PD4
		.db NONE,PIN3,PIN6,NONE	;servo 0=PD3	1=PB6
		.db PIN4,NONE,PIN2,NONE	;servo 0=PB4	1=PB2
		.db PIN0,NONE,NONE,PIN5	;servo 0=PB0	1=PD5


.macro RisePins
;Alza entrambi i pin di controllo del gruppo attivo.
;Per chiamare:	Z punta al gruppo attivo
;In uscita:	Z viene perso
;		R0 viene perso
;		R1 viene inizializzato
;		R2 viene inizializzato
;		R3 viene inizializzato
;		R4 viene inizializzato
;		R16 viene perso
		in R16,PORTB		;in r16 lo stato della porta B
		lpm			;in R0 la maschera porta B servo 0
		mov r1,r0		;lo copia in R1
		adiw ZL,1		;punta Z al byte successivo
		com R0			;inverte la maschera
		or r16,r0		;alza il pin nella porta B
		out PORTB,r16		;aggiorna la porta B
		
		in R16,PORTD		;in r16 lo stato della porta D
		lpm			;in R0 la maschera porta D servo 0
		mov r2,r0		;lo copia in R2
		adiw ZL,1		;punta Z al byte successivo
		com R0			;inverte la maschera
		or r16,r0		;alza il pin nella porta D
		out PORTD,r16		;aggiorna la porta D
		
		in R16,PORTB		;in r16 lo stato della porta B
		lpm			;in R0 la maschera porta B servo 0
		mov r3,r0		;lo copia in R3
		adiw ZL,1		;punta Z al byte successivo
		com R0			;inverte la maschera
		or r16,r0		;alza il pin nella porta B
		out PORTB,r16		;aggiorna la porta B
		
		in R16,PORTD		;in r16 lo stato della porta D
		lpm			;in R0 la maschera porta D servo 0
		mov r4,r0		;lo copia in R4
		adiw ZL,1		;punta Z al byte successivo
		com R0			;inverte la maschera
		or r16,r0		;alza il pin nella porta D
		out PORTD,r16		;aggiorna la porta D
					;tempo di esecuzione: 40 tic
.endmacro



.macro FallPin0
;Abbassa il pin del primo servo del gruppo.
;Per chiamare:	R1 inizializzato da RisePins
;		R2 inizializzato da RisePins
;In uscita:	R16 viene perso
		in R16,PORTB		;in r16 lo stato della porta B
		and r16,r1		;abbassa il pin nella porta B
		out PORTB,r16		;aggiorna la porta B
		in R16,PORTD		;in r16 lo stato della porta D
		and r16,r2		;abbassa il pin nella porta D
		out PORTD,r16		;aggiorna la porta D
					;tempo di esecuzione: 6 tic
.endmacro



.macro FallPin1
;Abbassa il pin del secondo servo del gruppo.
;Per chiamare:	R3 inizializzato da RisePins
;		R4 inizializzato da RisePins
;In uscita:	R16 viene perso
		in R16,PORTB		;in r16 lo stato della porta B
		and r16,r3		;abbassa il pin nella porta B
		out PORTB,r16		;aggiorna la porta B
		in R16,PORTD		;in r16 lo stato della porta D
		and r16,r4		;abbassa il pin nella porta D
		out PORTD,r16		;aggiorna la porta D
					;tempo di esecuzione: 6 tic
.endmacro

		
