;
;   Slave MODBUS protocol implementation for AVR family.
;   Copyright (C) 2010  Alessandro Accattini
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;   GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
;*****************************************************************************
;
;
;

.include "modcrc-0.01.asm"	;modbus CRC function


;*****************************************************************************
;  Required settings
; To let serial module to work you have to define following values, all
;the register defined here must NOT be used by the rest of the program:
;
.equ	Clock80000	=1	; system clock = 8.0000 MHz
; .equ	Clock73728	=1	; system clock = 7.3728 MHz


.ifdef Clock80000
.equ	B2400		=207
.equ	B4800		=103
.equ	B9600		=51
.equ	B14400		=34
.equ	B19200		=25
.equ	B28800		=16
.equ	B38400		=12
.equ	B57600		=8
.equ	B76800		=6
.equ	B115200		=3

.equ	T2400		=256-114
.equ	T4800		=256-57
.equ	T9600		=256-29
.equ	T14400		=256-19
.equ	T19200		=256-14
.equ	T28800		=256-10
.equ	T38400		=256-7
.equ	T57600		=256-5
.equ	T76800		=256-4
.equ	T115200		=256-3
.endif

.ifdef Clock73728
.equ	B2400		=191
.equ	B4800		=95
.equ	B9600		=47
.equ	B14400		=31
.equ	B19200		=23
.equ	B28800		=15
.equ	B38400		=11
.equ	B57600		=7
.equ	B76800		=5
.equ	B115200		=3

.equ	T2400		=256-105
.equ	T4800		=256-53
.equ	T9600		=256-26
.equ	T14400		=256-18
.equ	T19200		=256-13
.equ	T28800		=256-9
.equ	T38400		=256-7
.equ	T57600		=256-4
.equ	T76800		=256-3
.equ	T115200		=256-3
.endif


.macro ModbusDirectionPin
;Definisce la porta e la linea per il controllo direzione del bus.
;Per chiamare:	ModbusDirectionPin port,pin_num
.equ DirControl		=1
.equ DirPort		=PORT@0
.equ DirPortDDR		=DDR@0
.equ DirPin		=@1
.endmacro



;*****************************************************************************
;  Baudrate settings
; Define the serial baudrate:

.equ	Baudrate	=B2400		;velocita' della seriale
.equ	TimerCounter	=T2400		;timeout in ricezione
.equ	ModbusBufSize	=20		;dimensione dei buffer TX ed RX


;*****************************************************************************
;  Bus direction handling
; Uncomment the line below to define a direction handling pin.
; ModbusDirectionPin B,0		;uncomment to define direction control


;*****************************************************************************



.DSEG

ModbusRxBuf:	.byte ModbusBufSize	;buffer di ricezione
ModbusTxBuf:	.byte ModbusBufSize	;buffer di trasmissione
ModbusRxPtr:	.byte 2			;puntatore al buffer di ricezione
ModbusTxPtr:	.byte 2			;puntatore al buffer di trasmissione
ModbusRxBytes:	.byte 1			;contatore bytes ricevuti
ModbusTxBytes:	.byte 1			;contatore bytes da trasmettere
ModbusAddr:	.byte 1			;device address sul bus
ModbusStat:	.byte 1			;stato del decoder modbus
.equ	RxCompl		=0		;bit 0: 1 se la ricezione e' finita





.CSEG


.macro EnableModbusDirCtrl
;Imposta il pin di controllo direzione del bus in uscita.
;Per chiamare:	nessun parametro in ingresso
;In uscita:	tutti i registri vengono conservati
.ifdef DirControl
		sbi DirPortDDR,DirPin	;pin controllo direzione in uscita
.endif		

.endmacro
.macro ModbusDirInput
;Imposta la direzione del bus in ingresso.
;Per chiamare:	nessun parametro in ingresso
;In uscita:	tutti i registri vengono conservati
.ifdef DirControl
		cbi DirPort,DirPin	;abbassa il controllo direzione
.endif		
.endmacro

.macro ModbusDirOutput
;Imposta la direzione del bus in uscita.
;Per chiamare:	nessun parametro in ingresso
;In uscita:	tutti i registri vengono conservati
.ifdef DirControl
		sbi DirPort,DirPin	;alza il controllo direzione
.endif		
.endmacro

.macro StartModbusTimer
;Resetta e avvia il conteggio del timer di controllo fine messagio.
;Per chiamare:	nessun parametro in ingresso
;In uscita:	r16 viene perso
		ldi r16,TimerCounter	;in r16 il valore per il contatore
		out TCNT0,r16		;imposta il contatore del timer
		ldi r16,0x05		;seleziona CK/1024 per il timer 0
		out TCCR0,r16
.endmacro

.macro StopModbusTimer
;Resetta e blocca il conteggio del timer di controllo fine messagio.
;Per chiamare:	nessun parametro in ingresso
;In uscita:	r16 viene perso
		clr r16			;ferma il timer di controllo
		out TCCR0,r16
		ldi r16,TimerCounter	;in r16 il valore per il contatore
		out TCNT0,r16		;imposta il contatore del timer
.endmacro



ModbusInit:
;Inizializza la seriale, il timer e le strutture necessarie alla gestione
;del protocollo.
;Per chiamare:	r16 contiene l'indirizzo nel device sul bus modbus
;In uscita:	nessun registro viene gantito
		sts ModbusAddr,r16	;salva il device address
		StopModbusTimer		;carica il timer e lo ferma
.ifdef AT90S2313
;configura il timer 0
		in r16,TIMSK		;in r16 il registro TIMSK
		ori r16,1<<TOIE0	;abilita l'overflow del timer 0
		out TIMSK,r16		;aggiorna il registro
;configura la seriale
		cbi DDRD,0		;mette RX in ingresso
		sbi PORTD,0		;attiva il pull-up su RX
		sbi DDRD,1		;mette TX in uscita
		;clr r16			;spegne la seriale
		;out UCR,r16
		ldi r16,BaudRate	;imposta la velocita' della seriale
		out UBRR,r16
		ldi r16,(1<<RXCIE)|(1<<TXCIE)|(1<<RXEN)|(1<<TXEN)
					;modalita' 8n1 RX/TX
		out UCR,r16		;accende la seriale
.endif
.ifdef ATmega8
;configura il timer 0
;configura la seriale
		;clr r16			;spegne la seriale
		;out UCSRB,r16
		ldi r16,low(BaudRate)	;imposta la velocita' della seriale
		out UBRRL,r16
		ldi r16,high(BaudRate)
		out UBRRH,r16
		ldi r16,0x00		;modalita' normale
		out UCSRA,r16
		ldi r16,(1<<RXCIE)|(1<<TXCIE)|(1<<RXEN)|(1<<TXEN)
					;modalita' 8n1 TX/RX
		out UCSRB,r16		;accende la seriale
		ldi r16,0x80|0x06	;imposta la modalita' 8n1 RX/TX
		out UCSRC,r16
.endif
		EnableModbusDirCtrl	;pin di controllo direzione in uscita
		ModbusDirInput		;bus in ingresso
		clr r16			;inizializza lo stato del ricevitore
		sts ModbusStat,r16
		sts ModbusRxBytes,r16	;inizializza il numero char ricevuti
		ldi r16,low(ModbusRxBuf)	;inizializza il puntatore RX
		sts ModbusRxPtr,r16
		ldi r16,high(ModbusRxBuf)
		sts ModbusRxPtr+1,r16
		ret			;termina



IntRx:
;Interrupt di ricezione seriale.
;I char ricevuti vengono inseriti nel buffer di ricezione.
;Resetta il timer controllo ricezione.
		in Save,SREG		;salva il registro di stato
		push r16		;salva i registri utilizzati
		push r17
		push XH
		push XL
		in r16,UDR		;legge il char arrivato
		lds r17,ModbusRxBytes	;in r17 il numero di char arrivati
		cpi r17,ModbusBufSize-1	;controlla il limite
		brsh ModbusReject	;se maggiore o uguale salta
		lds XL,ModbusRxPtr	;punta X al buffer di ricezione
		lds XH,ModbusRxPtr+1
		st X+,r16		;salva il char nel buffer
		inc r17			;aggiorna il contatore dei char
		sts ModbusRxBytes,r17
		sts ModbusRxPtr,XL	;aggiornail puntatore al buffer
		sts ModbusRxPtr+1,XH
ModbusReject:	
		StartModbusTimer	;avvia o resetta il timer
		pop XL			;recupera i registri
		pop XH
		pop r17
		pop r16
		out SREG,Save		;reimposta il registro di stato
		reti			;termina l'interrupt
		
		

IntTx:
;Interrupt di trasmissione seriale.
;Invia i char presenti nel buffer di trasmissione.
		in Save,SREG		;salva il registro di stato
		push r16		;salva i registri utilizzati
		push XH
		push XL
		lds XL,ModbusTxPtr	;in X il puntatore al buffer
		lds XH,ModbusTxPtr+1
		ld r16,X+		;preleva un char dal buffer TX
		out UDR,r16		;invia il char alla seriale
		sts ModbusTxPtr,XL	;aggiorna il puntatore al buffer TX
		sts ModbusTxPtr+1,XH
		lds r16,ModbusTxBytes	;in r16 il contatore di bytes to send
		dec r16			;decrementa il contatore
		sts ModbusTxBytes,r16	;aggiorna il contatore
		brne NoModbusTxEnd	;se nonzero salta
;disabilita l'interrupt UDR empty
.ifdef AT90S2313
		in r16,UCR		;disabilita l'interrupt UDRIE
		andi r16,~(1<<UDRIE)		
		out UCR,r16
.endif
.ifdef ATmega8
		in r16,UCSRB		;disabilita l'interrupt UDRIE
		andi r16,~(1<<UDRIE)		
		out UCSRB,r16
.endif
NoModbusTxEnd:	pop XL			;recupera i registri
		pop XH
		pop r16
		out SREG,Save		;reimposta il registro di stato
		reti			;termina l'interrupt



IntTxComplete:
;Interrupt trasmissione seriale completata: se e' stato definito un pin
;per il controllo di direzione riposiziona la seriale in ricezione, altrimenti
;viene ignorato.
		in Save,SREG		;salva il registro di stato
		ModbusDirInput		;bus in ingresso
		out SREG,Save		;reimposta il registro di stato
		reti			;termina l'interrupt


IntTimer0:
;Interrupt timer di controllo ricezione: genera la fine della ricezione dopo
;una pausa di 1.5 byte time.
;La fine del messaggio viene generata sull'arrivo dell'interrupt, pertanto
;il timeout e' identico al tempo di roll-over del timer e il reset viene
;fatto direttamente sul timer stesso.
		in Save,SREG		;salva il registro di stato
		push r16		;salva i registri utilizzati
		lds r16,ModbusStat	;in r16 lo stato del decoder modbus
		ori r16,1<<RxCompl	;setta il flag
		sts ModbusStat,r16	;aggiorna lo stato del decoder
		StopModbusTimer		;ferma il timer
		pop r16			;recupera i registri
		out SREG,Save		;reimposta il registro di stato
		reti			;termina l'interrupt


ModbusStartTx:
;Avvia la trasmissione della stringa contenuta nel TX buffer.
;Per chiamare:	r16 numero di char da trasmettere
;In uscita:	r16 viene perso
		ModbusDirOutput		;bus in uscita
		dec r16			;toglie il primo byte dal conteggio
		sts ModbusTxBytes,r16	;salva il numero di bytes to send
		ldi r16,low(ModbusTxBuf+1)	;punta al buffer TX
		sts ModbusTxPtr,r16
		ldi r16,high(ModbusTxBuf+1)
		sts ModbusTxPtr+1,r16
		lds r16,ModbusTxBuf	;in r16 il primo char
		out UDR,r16		;invia il primo char
;abilita l'interrupt UDR empty
.ifdef AT90S2313
		in r16,UCR		;abilita l'interrupt UDRIE
		ori r16,1<<UDRIE		
		out UCR,r16
.endif
.ifdef ATmega8
		in r16,UCSRB		;abilita l'interrupt UDRIE
		ori r16,1<<UDRIE
		out UCSRB,r16
.endif
		ret			;termina


ModbusSlaveLoop:
;Chiamata ciclicamente dal main loop esegue i comandi ricevuti dal master.
;Per chiamare:	nessun parametro in ingresso
;In uscita:	tutti i registri vengono conservati
;controla se c'e' un messaggio in arrivo
		push r16		;salva r16
		lds r16,ModbusStat	;in r16 lo stato del decoder modbus
		bst r16,RxCompl		;in T il flag RX complete
		brts ModbusDecode	;se T=1 salta
		pop r16			;recupera r16
		ret			;termina
ModbusAbort:
		ldi r16,low(ModbusRxBuf)	;inizializza il puntatore RX
		sts ModbusRxPtr,r16
		ldi r16,high(ModbusRxBuf)
		sts ModbusRxPtr+1,r16
		clr r16
		sts ModbusRxBytes,r16	;inizializza il numero char ricevuti
		pop YL			;recupera i registri
		pop YH
		pop XL
		pop XH
		pop r19
		pop r18
		pop r17
		pop r16			;recupera r16
		ret			;termina
;decodifica il messaggio ricevuto
ModbusDecode:
		clr r16			;resetta lo stato del decoder modbus
		sts ModbusStat,r16
		push r17		;salva i registri utilizzati
		push r18
		push r19
		push XH
		push XL
		push YH
		push YL
;verifica il CRC del messaggio ricevuto
		ldi XL,low(ModbusRxBuf)	;punta X al comando ricevuto
		ldi XH,high(ModbusRxBuf)
		lds r18,ModbusRxBytes	;in r18 la lunghezza della stringa
		rcall ModbusCrc16	;in r17:r16 il crc
		or r16,r17		;controlla che il CRC sia 0
		brne ModbusAbort	;se non zero salta
;verifica il device address
		ldi YL,low(ModbusRxBuf)	;punta Y al comando ricevuto
		ldi YH,high(ModbusRxBuf)
		ld r16,Y+		;in r16 il device address nel messaggio
		lds r17,ModbusAddr	;in r17 il nostro device address
		cp r16,r17		;controlla se sono uguali
		brne ModbusAbort	;se diversi salta
		sts ModbusTxBuf,r16	;inserisce l'address nella risposta
;controlla il codice comando
		ld r16,Y+		;in r16 il codice comando ricevuto
		ld XH,Y+		;in X l'address che segue il comando
		ld XL,Y+
		sts ModbusTxBuf+1,r16	;inserisce il comando nella risposta
		cpi r16,3		;verifica se e' un comando 03
		breq ModbusCmd03	;se uguale salta
		cpi r16,4		;verifica se e' un comando 04
		breq ModbusCmd03	;se uguale salta
		cpi r16,6		;verifica se e' un comando 06
		breq ModbusCmd06	;se uguale salta
		cpi r16,16		;verifica se e' un comando 16
		breq ModbusCmd16	;se uguale salta
		rjmp ModbusAbort	;comando non riconosciuto
;esegue il comando 03: read of N words
; ModbusTxBuf contiene il device address
; ModbusTxBuf+1 contiene il codice comando
; Y punta al comando ricevuto (dopo il memory address relativo al comando)
; X punta in memoria all'indirizzo interessato dal comando
ModbusCmd03:
		ld r18,Y+		;in r18 WordCounterHIGH
		ld r18,Y+		;in r18 WordCounterLOW
		ldi YL,low(ModbusTxBuf+2)	;punta Y al buffer TX
		ldi YH,high(ModbusTxBuf+2)
		mov r19,r18		;in r19 il numero di words
		lsl r19			;in r19 il numero di bytes
		st Y+,r19		;inserisce il num. di bytes nella risp.
		ldi r19,3		;in r19 la lunghezza della risposta
ModbusReadLoop:	ld r16,X+		;in r17:r16 una word dalla memoria
		ld r17,X+
		st Y+,r17		;inserisce una word nella risposta
		st Y+,r16
		subi r19,-2		;aggiorna la dimensione della risposta
		dec r18			;decrementa il contatore delle words
		brne ModbusReadLoop	;se non zero salta
ModbusAddCrc:	mov r18,r19		;in r18 la dimensione della risposta
		ldi XL,low(ModbusTxBuf)	;punta X alla risposta
		ldi XH,high(ModbusTxBuf)
		rcall ModbusCrc16	;in r17:r16 il CRC della risposta
		st Y+,r16		;inserisce il CRC nella risposta
		st Y+,r17
		subi r19,-2		;aggiunge la dim. del CRC
		mov r16,r19		;in r16 la dimensione della risposta
		rcall ModbusStartTx	;invia la risposta al master
		rjmp ModbusAbort
;esegue il comando 06: write of single word
; ModbusTxBuf contiene il device address
; ModbusTxBuf+1 contiene il codice comando
; Y punta al comando ricevuto (dopo il memory address relativo al comando)
; X punta in memoria all'indirizzo interessato dal comando
ModbusCmd06:
		ld r17,Y+		;in r17 word-H
		ld r16,Y+		;in r16 word-L
		ldi YL,low(ModbusTxBuf+2)	;punta Y al buffer TX
		ldi YH,high(ModbusTxBuf+2)
		st Y+,XH		;inserisce l'address nella risposta
		st Y+,XL
		st Y+,r17		;inserisce la word nella risposta
		st Y+,r16
		st X+,r16		;inserisce la word nella RAM
		st X+,r17
		ldi r19,6		;in r19 la lunghezza della risposta
		rjmp ModbusAddCrc
;esegue il comando 16: write of N words
; ModbusTxBuf contiene il device address
; ModbusTxBuf+1 contiene il codice comando
; Y punta al comando ricevuto (dopo il memory address relativo al comando)
; X punta in memoria all'indirizzo interessato dal comando
ModbusCmd16:
		sts ModbusTxBuf+2,XH	;inserisce l'address H
		sts ModbusTxBuf+3,XL	;inserisce l'address L
		ld r17,Y+		;in r17 num-word-H
		ld r18,Y+		;in r16 num-word-L
		sts ModbusTxBuf+4,r17	;inserisce num-word-H
		sts ModbusTxBuf+5,r18	;inserisce num-word-L
		ld r16,Y+		;salta il numero di bytes a seguire
ModbusWriteLoop:
		ld r17,Y+		;in r17:r16 una word
		ld r16,Y+
		st X+,r16		;la scrive in RAM
		st X+,r17
		dec r18			;decrementa il contatore delle word
		brne ModbusWriteLoop	;se non zero salta
		ldi YL,low(ModbusTxBuf+6)	;punta Y al buffer TX
		ldi YH,high(ModbusTxBuf+6)
		ldi r19,6		;in r19 la lunghezza della risposta
		rjmp ModbusAddCrc


