;
;   12 channel servo-motor controller with serial interface.
;   Copyright (C) 2006 2008 2010 2012  Alessandro Accattini
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
;*****************************************************************************
;
; 0.05	Replaced comunication protocol with ModBus.
; 0.01	This release supports only AT90S2313.
;
;

.include "2313def.inc"			;AT90S2313
.equ		Clock8		=1	;quarzo da 8.0000MHz



.CSEG
;vettori di interruzione
		.org 0x0000
		rjmp Reset		;reset
		rjmp UnusedInt		;external interrupt 0
		rjmp UnusedInt		;external interrupt 1
		rjmp UnusedInt		;timer 1 capture
		rjmp Timer1		;timer 1 compare match
		rjmp UnusedInt		;timer 1 overflow
		rjmp IntTimer0		;timer 0 overflow
		rjmp IntRx		;uart RX complete
		rjmp IntTx		;uart data register empty
		rjmp IntTxComplete	;uart TX complete
		rjmp UnusedInt		;analog comparator




.DSEG

		.org 0x0060		;inizio RAM
;inizio registri modbus
Ctrl:		.byte 2			;registro di controllo

Dest0:		.byte 1			;destinazione servo 0
Dest1:		.byte 1			;destinazione servo 1
Dest2:		.byte 1			;destinazione servo 2
Dest3:		.byte 1			;destinazione servo 3
Dest4:		.byte 1			;destinazione servo 4
Dest5:		.byte 1			;destinazione servo 5
Dest6:		.byte 1			;destinazione servo 6
Dest7:		.byte 1			;destinazione servo 7
Dest8:		.byte 1			;destinazione servo 8
Dest9:		.byte 1			;destinazione servo 9
Dest10:		.byte 1			;destinazione servo 10
Dest11:		.byte 1			;destinazione servo 11

Spd0:		.byte 1			;velocita' servo 0
Spd1:		.byte 1			;velocita' servo 1
Spd2:		.byte 1			;velocita' servo 2
Spd3:		.byte 1			;velocita' servo 3
Spd4:		.byte 1			;velocita' servo 4
Spd5:		.byte 1			;velocita' servo 5
Spd6:		.byte 1			;velocita' servo 6
Spd7:		.byte 1			;velocita' servo 7
Spd8:		.byte 1			;velocita' servo 8
Spd9:		.byte 1			;velocita' servo 9
Spd10:		.byte 1			;velocita' servo 10
Spd11:		.byte 1			;velocita' servo 11

Pos0:		.byte 1			;posizione servo 0
Pos1:		.byte 1			;posizione servo 1
Pos2:		.byte 1			;posizione servo 2
Pos3:		.byte 1			;posizione servo 3
Pos4:		.byte 1			;posizione servo 4
Pos5:		.byte 1			;posizione servo 5
Pos6:		.byte 1			;posizione servo 6
Pos7:		.byte 1			;posizione servo 7
Pos8:		.byte 1			;posizione servo 8
Pos9:		.byte 1			;posizione servo 9
Pos10:		.byte 1			;posizione servo 10
Pos11:		.byte 1			;posizione servo 11

OffsetReg:	.byte 2			;offset posizione di zero
TimingReg:	.byte 2			;registro di controllo timing
;fine registri modbus

.equ	Dest_Off	=0		;offset destinazione
.equ	Speed_Off	=12		;offset velocita'
.equ	Pos_Off		=12+12		;offset posizione





.include "define.asm"			;define vari
.include "io.asm"			;accesso all'i/o
.include "timer.asm"			;timer di sistema
.include "modbus-0.01.asm"		;ModBus protocol





.CSEG


UnusedInt:	nop			;serve l'interrupt
		reti			;termina l'interrupt

;inizializzazione del micro
Reset:		cli			;disabilita le interruzioni
		ldi r16,low(Stack)	;punta lo stack in fondo alla RAM
		out SPL,r16
;inizializza i 12 servomotori
		ldi YH,high(Ctrl)	;punta Y al registro Ctrl
		ldi YL,low(Ctrl)
		clr r16			;clear r16
		st Y+,r16		;azzera il registro Ctrl
		st Y+,r16
		ldi r16,StartPos	;in r16 la posizione iniziale
		ldi r18,0xa0		;in r18 la velocita' iniziale
		ldi r17,12		;carica il contatore dei servomotori
InitLoop:	
		std Y+Dest_Off,r16	;destinazione
		std Y+Speed_Off,r18	;velocita'
		std Y+Pos_Off,r16	;posizione
		adiw YL,1		;next servo
		dec r17			;scala il contatore
		brne InitLoop		;se non zero salta
;inizializza i registridi controllo speciali
		ldi r16,low(ZeroOffset)	;in r16 la parte bassa
		sts OffsetReg,r16
		ldi r16,high(ZeroOffset)	;in r16 la parte alta
		sts OffsetReg+1,r16
		ldi r16,low(TimeDelay)	;in r16 la parte bassa
		sts TimingReg,r16
		ldi r16,high(TimeDelay)	;in r16 la parte alta
		sts TimingReg+1,r16
;inizializza il sistema
		rcall IoInit		;inizializza le porte di I/O
		rcall TimerInit		;inizializza il timer
		ldi r16,0x01		;in r16 il device address
		rcall ModbusInit	;inizializza il protocllo Modbus
;abilita le interruzioni
		sei			;abilita le interruzioni
;attende un messaggio dalla seriale
WaitCmd:	rcall ModbusSlaveLoop	;esegue i comandi in arrivo
		rjmp WaitCmd


