;
;   This is part of servo.
;   Timer interrupt routine: generates output pulses periodically.
;   Copyright (C) 2006 2012  Alessandro Accattini
;
;   This program is free software; you can redistribute it and/or modify
;   it under the terms of the GNU General Public License as published by
;   the Free Software Foundation; either version 2 of the License, or
;   (at your option) any later version.
;
;   This program is distributed in the hope that it will be useful,
;   but WITHOUT ANY WARRANTY; without even the implied warranty of
;   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;   You should have received a copy of the GNU General Public License
;   along with this program; if not, write to the Free Software
;   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;
;*****************************************************************************
;
;
;

.DSEG

Couple:		.byte 1			;coppia di servi emessa



.CSEG

TimerInit:
;Inizializza il timer 1 per interrompere ogni 20ms.
;Per chiamare:	nessun parametro in ingresso
;In uscita:	nessun registro viene conservato
		ldi r16,0x00		;prima coppia emessa
		sts Couple,r16
		ldi r16,high(TimeRate)	;imposta l'OCR per il timer 1
		out OCR1AH,r16
		ldi r16,low(TimeRate)
		out OCR1AL,r16
		ldi r16,0x00		;disabilita il generatore PWM
		out TCCR1A,r16
		ldi r16,0x0d		;abilita compare match con CK/1024
		out TCCR1B,r16		;clk=7812.5Hz
		ldi r16,0x40		;abilita l'interruzione del timer
		out TIMSK,r16
		ret			;termina
		


Timer1:
;Timer interrupt handler.
;Questa interruzione viene generata una volta ogni 20ms.
;Genera un impulso per ogni servo nella coppia attuale.
		in Save,SREG		;copia lo status register
		push r16		;salva i registri utilizzati
		push r17
		push ZH
		push ZL
		push YH
		push YL
		push XH
		push XL

;individua la coppia attuale
		lds r16,Couple		;in r16 la coppia attuale
		mov r17,r16		;copia il valore in r17
		inc r17			;in R17 la prossima coppia
		cpi r17,6		;controlla la fine giro
		brlo NoRoll		;se minore salta
		clr r17			;ricomincia il giro
NoRoll:		sts Couple,r17		;aggiorna Couple
		lsl r16			;moltiplica per 2
		mov r17,r16		;lo copia in r17
		lsl r17			;in r17 la coppia *4
		ldi YH,high(Dest0)	;punta Y al primo servo
		ldi YL,low(Dest0)
		ldi ZH,high(ServoPin*2)	;punta Z ai pin della prima coppia
		ldi ZL,low(ServoPin*2)
		add YL,r16		;somma la parte bassa
		clr r16			;clear register
		adc YH,r16		;punta Y alla coppia attuale
		add ZL,r17		;somma la parte bassa
		adc ZH,r16		;punta Z ai pin della coppia attuale
		push YL			;salva Y
		push YH
;carica i contatori della coppia attuale
		ldd XL,Y+Pos_Off	;in X il contatore servo 0
		clr XH
		ldd YL,Y+Pos_Off+1	;in Y il contatore servo 1
		clr YH
		lds r16,OffsetReg	;in r17:r16 l'offset iniziale
		lds r17,OffsetReg+1
		add XL,r16		;aggiunge l'offset iniziale
		adc XH,r17
		add YL,r16		;aggiunge l'offset iniziale
		adc YH,r17
;loop generazione impulsi
		ldi r17,2		;carica il contatore pin alti
		RisePins		;alza entrambi i pin di controllo
		
PulseLoop:	sbiw XL,1		;dec X, contatore servo 0
		brne Pulse0		;se non zero salta
		FallPin0		;abbassa il pin 0 (6 tic)
		dec r17			;scala un pin
		rjmp NoDelay0
Pulse0:		push r16		;perde 8 tic
		pop r16
		push r16
		pop r16
NoDelay0:	
		sbiw YL,1		;dec Y, contatore servo 1
		brne Pulse1		;se non zero salta
		FallPin1		;abbassa il pin 1 (6 tic)
		dec r17			;scala un pin
		rjmp NoDelay1
Pulse1:		push r16		;perde 8 tic
		pop r16
		push r16
		pop r16
NoDelay1:
;arrivato fin qui sono necessari 5 tic per pareggiare il loop
;piu' altri a richiesta del Timing register.
;Il loop perde esattamente: (Timing-1)*4+5 tic
		lds r16,TimingReg	;in r16 la parte bassa del registro
TimingLoop:	nop
		dec r16			;scala il contatore
		brne TimingLoop		;se non zero salta
		
		tst r17			;verifica il numero segnali alti
		brne PulseLoop		;se non zero salta

;sposta i servi della coppia attuale
		pop YH			;punta Y al primo servo della coppia
		pop YL
		rcall NewPos		;muove il servo 0
		adiw YL,1		;punta Y al servo succesivo
		rcall NewPos		;muove il servo 1

;recupera i registri
		pop XL			;recupera i registri salvati
		pop XH
		pop YL
		pop YH
		pop ZL
		pop ZH
		pop r17
		pop r16
		out SREG,Save		;reimposta lo status register
		reti			;termina


NewPos:
;Determina la nuova posizione di un servo.
;Per chiamare:	Y punta alla posizione del servo interessato
;In uscita:	Y viene coservato
;		R16 viene perso
;		R17 viene perso
;		XL viene perso
		ldd r17,Y+Dest_Off	;in r17 la destinazione servo
		ldd XL,Y+Speed_Off	;in XL la velocita' massima
		ldd r16,Y+Pos_Off	;in r16 la posizione dervo
		sub r17,r16		;in r17 lo spostamento necessario
		breq NoMove		;se uguali salta
		brsh Avanti		;se maggiore o uguale salta
;muove il servo indietro
		neg r17			;cambia segno allo spostamento
		cp XL,r17		;confronta spostamento e velocita' max
		brsh NoLim		;se XL >= r17 salta
		mov r17,XL		;usa la velocita' massima consentita
NoLim:		neg r17			;rimette il segno negativo originale
		rjmp MoveServo
;muove il servo 0 avanti
Avanti:		cp XL,r17		;confronta spostamento e velocita' max
		brsh MoveServo		;se XL >= r17 salta
		mov r17,XL		;usa la velocita' massima consentita
MoveServo:	add r16,r17		;sposta il servo 0		
		std Y+Pos_Off,r16	;aggiorna la posizione del servo 0
NoMove:		ret			;termina


